@@ -3,6 +3,12 @@
 /* This program may be used without any warranty. It may be modified and */
 /* distributed without any restrictions.                                 */
 
+
+/*
+   This file is modified for use with Solaris. However, it shoud go
+   with any XWindow ...
+*/
+
              
 #include <stdio.h>
 #include <X11/Xos.h>                      /* X header files              */ 
@@ -13,13 +19,21 @@
 
 #include <math.h>
 
-#include "defslx.h"
+
+#include "../src/defs.h"
+
 
 /* prefered Image format */
 /* you may set up tmview for the depth of your display. 8bit and 16bit is  */
 /* tested. If you use tmview with a depth not the same as defined here, it */
 /* will become very slow. */
 
+/* 32/24 bit per pixel */
+/*#define IUNIT unsigned long 
+#define IBYTES_PER_PIXEL 4
+#define IBITS_PER_PIXEL 32
+#define IGET_PIXEL(ptr) (*(ptr))
+#define ISET_PIXEL(ptr,c) (*(ptr) = (c))*/ 
 /* 16 bit per pixel */
 /*#define IUNIT unsigned short 
 #define IBYTES_PER_PIXEL 2
@@ -31,7 +45,7 @@
 #define IBYTES_PER_PIXEL 1
 #define IBITS_PER_PIXEL 8
 #define IGET_PIXEL(ptr) (*(ptr))
-#define ISET_PIXEL(ptr,c) (*(ptr) = (c))
+#define ISET_PIXEL(ptr,c) (*(ptr) = (c)) 
 
 
 /* #define DEBUGX  /* watch tmview contacting the XServer */
@@ -133,7 +147,8 @@ void alloc_off(void) {
   offimage->data=offscreen;
   ibytes_per_line=offimage->bytes_per_line;
   ibytes_per_pixel=offimage->depth >> 3;
-
+  if(ibytes_per_pixel == 3) ibytes_per_pixel =4;
+  
   if(ibytes_per_pixel != IBYTES_PER_PIXEL) {
     pfprot("\nwarning: tmVIEW prefers %d-bit depth. Recompile for better performance.\n", IBYTES_PER_PIXEL*8);
     setvar();
@@ -370,15 +385,36 @@ void vgaopen(void) {
 
   /* show resources */
 #ifdef DEBUGX
-  pfprot("(rmfstr\n %s\n)\n\n",XResourceManagerString(dpy));
+  pfprot("(rmfstr\n %s\n)\n\n",XResourceManagerString(dpy)); 
 #endif
 
   tfontname=XGetDefault(dpy,"DviLX","sfont");
+  if(tfontname==NULL)
+    tfontname="-b&h-lucidatypewriter-medium-*-*-*-14-*-*-*-*-*-*-*";
+
   tforename=XGetDefault(dpy,"DviLX","sforeground");
+  if(tforename==NULL)
+    tforename="tomato4";
+
   tbackname=XGetDefault(dpy,"DviLX","sbackground");
+  if(tbackname==NULL)
+    tbackname="OldLace";
+
   dblackname=XGetDefault(dpy,"DviLX","foreground");
-  dwhitename=XGetDefault(dpy,"DviLX","background");
+  if(dblackname==NULL)
+    dblackname="black";
+
+  dwhitename=XGetDefault(dpy,"","DviLX.background");
+  if(dwhitename==NULL)
+    dwhitename=XGetDefault(dpy,"OpenWindows","DataBackground");
+  if(dwhitename==NULL)
+    dwhitename="OldLace";
+
   geometry=XGetDefault(dpy,"DviLX","geometry");
+  /*  if(geometry==NULL)
+    geometry="500x500"; */
+
+  
 
 #ifdef DEBUGX
 
@@ -387,7 +423,8 @@ void vgaopen(void) {
   pfprot("(getdef tback  %s)\n",tbackname);
   pfprot("(getdef dwhite %s)\n",dwhitename);
   pfprot("(getdef dblack %s)\n",dblackname);
-  pfprot("(getdef geom   %s)\n",geometry);
+  if(geometry!=NULL)
+    pfprot("(getdef geom   %s)\n",geometry);
 
   /* Get some info */
   depthlist=XListDepths(dpy,screen,&n);
@@ -1193,7 +1230,7 @@ uchar vgagetchar(int block) {
     truevgaydim=newydim;
     vgaxdim=newxdim;
     vgaydim=newydim;
-    pfprot("(resize %d %d)\n",vgaxdim,vgaydim); 
+    /* pfprot("(resize %d %d)\n",vgaxdim,vgaydim); */
     free_off();
     alloc_off(); 
     vgamaxstatuslines=(truevgaydim-3)/vgastatushight;
@@ -1204,11 +1241,13 @@ uchar vgagetchar(int block) {
     break;
   case ButtonPress:
     butev = (XButtonEvent *) &event;
+    if(butev->button ==2) {mouseleft=1;mouseright=1;}
     if(butev->button ==1) mouseleft=1;
     if(butev->button ==3) mouseright=1;
     break;      
   case ButtonRelease:
     butev = (XButtonEvent *) &event;
+    if(butev->button ==2) {mouseleft=0;mouseright=0;}
     if(butev->button ==1) mouseleft=0;
     if(butev->button ==3) mouseright=0;
     break;      
