@@ -1,8 +1,8 @@
 #define HELPWIDTH 60 
 #define HELPSTR "\
-LIST OF COMMANDS ******************************************
+LIST OF COMMANDS *********************************** V 98.06
 
-<i>/<m>           select a page relative to the current page 
+<i>/<m>           select a page relative to the current page
 <g>               select a page w.r.t. TeX-counters
 <u>/<n>/<h>/<j>   scroll the visible area
 <f>/<c>           make scrolling finer/coarser
@@ -542,7 +542,10 @@ MISC *******************************************************
       Ooc                       not found
       
   
-<r> Re-read current DVIfile and re-draw screen
+<r> Re-read current DVIfile and re-draw screen.
+
+      Note: <*><r> will re-initialize the fontdatabase too.
+
 
 <d> Load/kill DVIfile
 
