@@ -13,13 +13,15 @@
 #include "subs.h"
 #include "readtfm.h"
 #include "readpk.h"
+#include "readvf.h"
 #include "halfhyp.h"
 #include "drawit.h"
 #include "readdvi.h"
 
 
-/* #define DEBUGDVI */
-/* #define DEBUGFONTDATABASE */
+/*#define DEBUGDVI */
+/*#define DEBUGFONTDATABASE*/
+/*#define DEBUGMACRO */
 
 
 
@@ -77,6 +79,7 @@ int allocfontdesc(char* ffpath, char* ffname, long dvimag, long scf, long dsz) {
   font->foscf = scf;
   font->fodsz = dsz;
   font->pkfile = NULL;
+  font->vffile = NULL;
   font->encoding = NULL; 
   font->mch = -1;
   font->usecount=0;
@@ -93,7 +96,7 @@ int searchfontdatabase(char* ffpath, char* ffname,
   fontdesc* font;
   int i;
 #ifdef DEBUGFONTDATABASE
-  pfprot("(searchfontdatabase %s %s %6.1f ",ffpath,ffname,fmag);
+  pfprot("(searchfontdatabase %s %s %6.1f ",ffpath,ffname,dvimag);
 #endif
   for(i=0, font=fontdatabase;i<nfontdatabase;i++,font++) {
     if(font->usecount<0) continue;
@@ -133,8 +136,12 @@ void cleanfontdatabase(void) {
           freelrumem(&(font->chv[c].bmp.bits));  
           freelrumem(&(font->chv[c].bmp2.bits));  
         }
+        if (font->chv[c].fty & VFTYPE) {
+          freemem(&(font->chv[c].macro));  
+        }
       freemem(&(font->chv));
       freemem(&(font->pkfile));
+      freemem(&(font->vffile));
       freemem(&(font->fonam));
       freemem(&(font->fopath));
       font->usecount=-1;
@@ -280,14 +287,15 @@ void dvipositpost(void) {
 
 
 /* local globals for dvi-file-processing */
+/* (only dvimag is real global, since it is used by readvf.c) */
 
 static long hs[SMAX], vs[SMAX], ws[SMAX], xs[SMAX], ys[SMAX], zs[SMAX];
 static char comment[257], dvicomment[257];
 
 static double conv, vconv, sconv, svconv;
-static long  dvimag ,a, b, q, w, x, y, z, h, v;
+static long  /*dvimag ,*/a, b, q, w, x, y, z, h, v;
 static long  mag, spl,  mxh, mxv, lastpage;
-static int k;
+static int k, fint;
 
 static ushort s, mst;
 
@@ -311,8 +319,7 @@ void fontdef(long fontnr) {
   p = dvibyte(); 
   q = dvibyte();  
   dvibytes(fontname, q);
-  if (p == 0)  strcpy(ffpath, fontprefix);  
-  else dvibytes(ffpath, p); 
+  if (p != 0)  dvibytes(ffpath, p); 
   fontmag = xres * (dvimag / 1000.0 * ((double)scf / dsz));
   pfprot("preparing font: %8s at %6.1f ", fontname, fontmag);
   pfverb("(scf %d dsz %d) ",scf,dsz);
@@ -320,25 +327,42 @@ void fontdef(long fontnr) {
   sprintf(messtr,"%10s at %6.1f",fontname,fontmag);
   if(proceedthis(messtr)) {dvibad("user interrupt"); return;}
 
-  if((fdb=searchfontdatabase(ffpath,fontname,dvimag,scf,dsz))!=-1) {
-    pfprot(" --- found in fontdatabase");
-    font=fontdatabase+fdb;
+  fdb=-1;
+  if (p==0) {
+    if((fdb=searchfontdatabase(vfprefix,fontname,dvimag,scf,dsz))==-1)
+      fdb=searchfontdatabase(fontprefix,fontname,dvimag,scf,dsz);
   } else {
+    fdb=searchfontdatabase(ffpath,fontname,dvimag,scf,dsz);
+  }
+  if(fdb!=-1) {
+    pfprot(" - fontdatabase");
+    font=fontdatabase+fdb;
+    if (font->vffile!=NULL) /* look for involved pks */
+      vflookuppk(fdb,fontnr);
+  } else {
+    if (p==0) strcpy(ffpath, vfprefix);
     fdb=allocfontdesc(ffpath,fontname,dvimag,scf,dsz);
     font=fontdatabase+fdb;  
-    font->pkfile = NULL;
     allocmem(&(font->chv),(MAXFONTS+1) * sizeof(chdesc)); 
     memset(font->chv,0,(MAXFONTS+1) * sizeof(chdesc));  
                                                /* unsused bitmaps=NULL */
                                                /* unused  fty  0 */
                                                /* unused dimensions 0*/
-    if(pkfind(ffpath,fontname,fontmag,fftuto))
-      pkdef(fftuto,fdb,chksum);
+                                               /* unused macros = 0*/
     if(tfmfind(tfmprefix,fontname,fftuto))
       tfmdef(fftuto,fdb,chksum);
+    if(vffind(ffpath,fontname,fftuto)) {
+      vfdef(fftuto,fdb,chksum,fontnr);
+    } else {
+      if (p==0) strcpy(ffpath, fontprefix);
+      if(pkfind(ffpath,fontname,fontmag,fftuto)) {
+        pkdef(fftuto,fdb,chksum);
+	stralloccpy(&(font->fopath),ffpath);
+      }
+    }
     reallocmem(&(font->chv),sizeof(chdesc)*(font->mch + 2));
                                             /* +1 is enuough ?? */
-  }
+  }  
   f = fontnr & 0xff;  a = 0;
   if ((fontnr >= MAXFONTS) || (fontvect[f].fontdataptr != -1)) do {
     if (fontvect[a].fontdataptr == -1)  f = a;
@@ -359,7 +383,7 @@ void fontdef(long fontnr) {
 }
 
 
-uchar getintnr(long fontnr) {   
+int getintnr(long fontnr) {   
   int a, f, atleast;
   f = fontnr & 0xff;  
   if (fontvect[f].fontdataptr != -1) 
@@ -371,7 +395,7 @@ uchar getintnr(long fontnr) {
       else if (fontvect[a].dfn != fontnr) atleast=a++;
         else return(a);
   } while (a < MAXFONTS);
-  pfprot("\n %d ->",fontnr);
+  pfprot("\n %u ->",fontnr);
   dvibad("is not defined as a fontnumber");
   return(atleast);
 }
@@ -602,6 +626,8 @@ void killdvi(void) {
 }
 
 
+int  runvfmacro(chdesc* thechar);  /* forward */
+
 static void listchar(int c,int fontintnr) {
   
   fontdesc *font;  
@@ -611,7 +637,7 @@ static void listchar(int c,int fontintnr) {
 
   font = fontdatabase+fontvect[fontintnr].fontdataptr;
   if(font->mch < c) {
-    pfprot(
+    pfverb(
     "(putchar: warning: skipping undefined charakter %d in %s)",c,font->fonam);
     q = 0;
     return;
@@ -650,6 +676,26 @@ static void listchar(int c,int fontintnr) {
     (*data).chr.chdp=thechar; 
     return;
   } /* end pk */
+  if (thechar->fty & VFTYPE) {                            /* try vf */
+    if(thechar->macro==NULL) {
+       pfverb(
+        "(putchar: warning: skipping macro %d in %s)",c,font->fonam);
+       return;
+    }
+    /* Debugging code: draw at least something */
+    /*
+    /*br = LROUND(sconv * (thechar->tfw));
+    hr = LROUND(sconv * h);
+    vr = LROUND(svconv *v);
+    data=drawlistadd(*drawrulebg);
+    (*data).rect.c=TFMCOL;
+    (*data).rect.x=hr; (*data).rect.y=vr;
+    (*data).rect.w=br; (*data).rect.h=3;
+    */
+    runvfmacro(thechar);
+    q = thechar->tfw;    
+    return;    
+  } /* end vf */  
   if (thechar->fty & TFMTYPE) {                           /* try tfm */
     hr = LROUND(sconv * h);
     vr = LROUND(svconv * (v - thechar->tfmtfh));
@@ -675,7 +721,7 @@ static void listchar(int c,int fontintnr) {
     q = thechar->tfmtfw;
     return;  
   } /* end tfm */  
-  pfprot(
+  pfverb(
    "(putchar: warning: skipping undefined charakter %d in %s)",c,font->fonam);
   q = 0;
 }
@@ -761,9 +807,160 @@ static void listrule(void) {
 }
 
 
+/************************************************************/
+/* Macro processing: Striktly non-recursive, all global !!  */
+
+static long   macropos;
+static uchar* macrodata;
+static long   macrolen;
+static long   vffint;
+
+uchar macrobyte(void)  {  
+  if(macrolen-macropos < 1) return 0L;
+  return macrodata[macropos++];
+}
+
+
+long macronum(uchar b)  {  
+  uchar i;  
+  long n;
+  if(macrolen-macropos < b) return 0L;
+  n = 0;
+  for (i = 1; i <= b; i++)  
+    n = (n << 8) | (uchar) macrodata[macropos+i-1];
+  macropos+=b;
+  return n;
+}
+
+long macroint(uchar b)  {  
+  uchar i;  
+  long n;
+  if(macrolen-macropos < b) return 0L;
+  n = macrodata[macropos];
+  if (n > 127)  n -= 256;
+  for (i = 2; i <= b; i++)  
+    n = (n << 8) | macrodata[macropos+i-1];
+  macropos+=b;
+  return n;
+}
+
+
+int  runvfmacro(chdesc* thechar){
+  uchar o,c;
+  int i,ishr;
+  ushort d1,d2;
+  long f, vffnr, vffint;
+
+
+  macropos=0;
+  macrodata=thechar->macro;
+  macrolen=thechar->mlen;
+  vffint=fint;
+  vffnr=fontvect[vffint].dfn;
+  fint=getintnr((1+vffnr)*MAXFONTS + thechar->lastf);
+
+#ifdef DEBUGMACRO
+  pfprot("(run macro c %u in %s ... ",
+	 thechar->ch, fontdatabase[thechar->fontdataptr].fonam);
+  pfprot("macrodata at 0x%x macrolen %u : ",macrodata,macrolen);
+  for(i=0;i<macrolen;i++)
+    pfprot("%u ",macrodata[i]);
+#endif
+
+  if (s < SMAX) {
+       s++;  hs[s - 1] = h;  vs[s - 1] = v;
+       ws[s - 1] = w;	xs[s - 1] = x;	ys[s - 1] = y; zs[s - 1] = z;
+  } else  dvibad("macro: stack overflow");  
+  
+  q=0;
+  while (macropos<macrolen) {
+    o = macrobyte();
+    /* pfprot("(%d)",o); */
+    if (o <= 127) {
+       listchar(o,fint);  h += q;
+    } else if (o >= 171 && o <= 234) {
+	 fint=getintnr(o-171+(1+vffnr)*MAXFONTS);
+    } else
+    switch (o) {
+    case 128:  case 129:  case 130:  case 131:
+      c = macronum(o - 127);  listchar(c,fint);  h += q;  break;
+    case 132:  a = macroint(4);  b = macroint(4);
+      listrule();  h += b;  break;
+    case 133:  case 134:  case 135:  case 136:
+      c = macronum(o - 132);  listchar(c,fint);  break;
+    case 137:  a = macroint(4);  b = macroint(4);
+      listrule();  break;
+    case 138: break;
+    case 139: dvibad("macro: unexpectet bop"); break;
+    case 140: break;
+    case 141:  if (s < SMAX) {	s++;  hs[s - 1] = h;  vs[s - 1] = v;
+	ws[s - 1] = w;	xs[s - 1] = x;	ys[s - 1] = y; zs[s - 1] = z;
+      } else  dvibad("macro: stack overflow");  break;
+    case 142:  if (s > 0) {  h = hs[s - 1];  v = vs[s - 1];
+	w = ws[s - 1];	x = xs[s - 1];	y = ys[s - 1];	z = zs[s - 1];	s--;
+      } else  dvibad("macro: stack underflow");  break;
+    case 143:  case 144:  case 145:  case 146:
+      h += macroint(o - 142);  break;
+    case 147:  h += w;  break;
+    case 148:  case 149:  case 150:  case 151:
+      w = macroint(o - 147);  h += w;  break;
+    case 152:  h += x;  break;
+    case 153:  case 154:  case 155:  case 156:
+      x = macroint(o - 152);  h += x;  break;
+    case 157:  case 158:  case 159:  case 160:
+      v += macroint(o - 156);  break;
+    case 161:  v += y;  break;
+    case 162:  case 163:  case 164:  case 165:
+      y = macroint(o - 161);  v += y;  break;
+    case 166:  v += z;  break;
+    case 167:  case 168:  case 169:  case 170:
+      z = macroint(o - 166);  v += z;  break;
+    case 235:  case 236:  case 237:  case 238:
+      f = macronum(o - 234);
+      fint=getintnr(f+(1+vffnr)*MAXFONTS);
+      break;
+    case 239:  case 240:  case 241:  case 242:
+      spl = macronum(o - 238);
+      macropos+=spl;
+      pfprot("(ignoring special within macro)");
+      break;
+    case 243:  case 244:  case 245:  case 246:
+      f = macronum(o-242);
+      pfprot("(fontdef within macro ignored)"); 
+      macropos+=12L; 
+      d1 = macrobyte();  d2 = macrobyte();
+      macropos+=((long)(d1 + d2)); break;
+    case 250:  case 251:  case 252:
+    case 253:  case 254:  case 255:  
+      pfprot("(skipping unknown opc > 249 within macro)");break;
+    default:
+    } /* end switch o */
+  }
+
+  macropos=0;
+  macrolen=0;
+  fint=vffint;
+  if (s > 0) {
+    h = hs[s - 1];  v = vs[s - 1];
+    w = ws[s - 1];	x = xs[s - 1];	y = ys[s - 1];	z = zs[s - 1];	s--;
+  } else  dvibad("macro: stack underflow"); 
+  
+#ifdef DEBUGMACRO
+  pfprot("... macro)");
+#endif
+
+}
+
+
+    
+
+/*************************************/
+/* fnaly lets read a page ...        */
+
+
 int  readpage(short pn, float fshr, uchar col,int force){
   uchar o,c;
-  int fint,i,ishr;
+  int i,ishr;
   ushort d1,d2;
   long f;
 
@@ -1089,7 +1286,6 @@ void searchchar(int c,int fontintnr) {
 void searchreaddvi(int *spage, long *spos) {
   long startpos;
   uchar o,c;
-  int fint;
   ushort d1,d2;
   char messtr[20];
   long f;
@@ -1279,7 +1475,7 @@ void anchorcharq(int c,int fontintnr) {
 
 readanchors(int page) {
   uchar o,c;
-  int fint,i,j,an;
+  int i,j,an;
   ushort d1,d2;
   long f,startpos;
 
