@@ -204,7 +204,7 @@ int pkfind(char* ffpath,char* fontname,float fontmag,char* fftuto)  {
 #endif
 
   if(foundfile==0) { 
-        pfprot(" - no PKfile found "); 
+        pfprot(" - no PKfile "); 
         return(0);
   }
   fofile=fopen(fftuto,"rb");
@@ -213,7 +213,7 @@ int pkfind(char* ffpath,char* fontname,float fontmag,char* fftuto)  {
     fclose(fofile);
     return(0);
   }
-  pfprot(" - found PKfile ");
+  pfprot(" - PKfile ");
   pfverb("%s ",fftuto);
   fclose(fofile);
   return(1);
