@@ -597,8 +597,18 @@ int completefile(char* total, int which) {
 }
 
 void setworkdir(void) {
- workdir=getcwd(NULL, 0);
-  if(workdir==NULL) {pfprot("\nfatal error: memory .. getcwd ??");exit(1);}
+  char* buptr;
+  
+  /* correct version: linux or gnu-c resp. ... */
+  /*workdir=getcwd(NULL, 0);
+  if(workdir==NULL) {pfprot("\nfatal error: memory .. getcwd ??\n");exit(1);}*/
+  /* dirty hack, since getcwd is not allways gnu-ic ... */
+  allocmem(&buptr,2000);
+  buptr=getcwd(buptr, 2000);
+  buptr[2000]=0;
+  workdir=NULL;
+  stralloccpy(&workdir, buptr);
+  freemem(&buptr);
 }
 
 int dvistandard(char** dst,char* src) {  /* may be src==filename */
@@ -613,7 +623,7 @@ int dvistandard(char** dst,char* src) {  /* may be src==filename */
 #endif
   if(src[0]!=DIRSEPCHAR) {
     if(strlen(workdir)+strlen(src)>MAXPATHSTR) 
-       {pfprot("\nfatal error: memory .. stddvi ??");exit(1);}
+       {pfprot("\nfatal error: memory .. stddvi ??\n");exit(1);}
     strcpy(ss,workdir);
     strcat(ss,DIRSEPSTR);
     strcat(ss,src);
@@ -752,7 +762,7 @@ void setworkdir(void){
 
  _nls_init();  /* hack: do this here ... */
  workdir=_getcwd2(NULL,0);
- if(workdir==NULL) {pfprot("\nfatal error: memory .. getcwd ??");exit(1);}
+ if(workdir==NULL) {pfprot("\nfatal error: memory ..  getcwd ??\n");exit(1);}
  for(i=strlen(workdir)-1;i>=0;i--) 
    if(workdir[i]=='/') workdir[i]= DIRSEPCHAR; /* translate back */
 #ifdef DEBUGFILE
@@ -779,7 +789,7 @@ int dvistandard(char** dst,char* src) {  /* may be src==filename */
   else if(src[1]!=':') adw=1;
   if(adw) {
     if(strlen(workdir)+slen>MAXPATHSTR) 
-       {pfprot("\nfatal error: memory .. stddvi ??");exit(1);}
+       {pfprot("\nfatal error: memory .. stddvi ??\n");exit(1);}
     strcpy(ss,workdir);
     strcat(ss,DIRSEPSTR);
     strcat(ss,src);
