@@ -461,62 +461,68 @@ int helppos=0;
 void help(void) {
   uchar o;
   int h,w,i,j,k,l,dline,dpos;
-  vgasetstatuslines(vgamaxstatuslines);
-  h=vgastatuslines-3;
-  w=vgastatuslen;
-  dline=helpline;
-  dpos=helppos;
-  drawsl("",h,0,w);
-  drawsl("",h+1,0,w);
-  drawsl("",h+2,0,w);
-  drawsr("PRESS <U>/<N>/<H>/<J> TO MOVE OR <ESC> TO EXIT",h+2,w-46,46);
-  do {
-    if(w>HELPWIDTH) 
-      dpos=-(w-HELPWIDTH)/2;
-    helppos=dpos;
-    for(i=0,j=0; helpstr[i]!=0;i++) {
-      if(j==dline) break;
-      if(helpstr[i]==10) j++;
-    }
-    if(j==dline) helpline=dline;
-    for(i=0,j=0; helpstr[i]!=0;i++) {
-      if(j==helpline) break;
-      if(helpstr[i]==10) j++;
-    }
-    for(;helpstr[i]!=0 && j<helpline+h;j++) {
-      for(l=0;helpstr[i+l]!=10 && helpstr[i+l]!=0;l++);
-      k=i+l+1;
-      drawsl("",j-helpline,0,MAX(0,-helppos));
-      l=MIN(l,l-helppos);
-      drawsl(helpstr+i+MAX(0,helppos),j-helpline,MAX(0,-helppos),l); 
-      drawsl("",j-helpline,l+MAX(0,-helppos),w);
-      i=k;
-    }   
-    for(;j<helpline+h;j++)
-      drawsl("",j-helpline,0,w);
+  do{
+    vgasetstatuslines(vgamaxstatuslines);
+    h=vgastatuslines-3;
+    w=vgastatuslen;
     dline=helpline;
     dpos=helppos;
-    vgaupdatestatus();
+    drawsl("",h,0,w);
+    drawsl("",h+1,0,w);
+    drawsl("",h+2,0,w);
+    drawsr("PRESS <U>/<N>/<H>/<J> TO MOVE OR <ESC> TO EXIT",h+2,w-46,46);
     do {
-      o=vgagetchar(1);
-      if('a'<= o && o <= 'z') o=o-'a'+'A';
-      switch(o) {
-      case KEYUP: case 'U':
-        dline=MAX(0,dline-h/3); break;
-      case KEYDOWN: case 'N':
-        dline+=h/3; break;
-      case KEYRIGHT: case 'J':
-        dpos=MAX(0,MIN(dpos+w/3,HELPWIDTH-2.0/3*w)); break;
-      case KEYLEFT: case 'H':
-        dpos=MAX(-HELPWIDTH+2.0/3*w,dpos-w/3);
-      case KEYESC: break;
-      case KEYRESIZE: 
-        comdoresize();
-        break;
-      default: o=KEYNOP; break;
+      if(w>HELPWIDTH) 
+        dpos=-(w-HELPWIDTH)/2;
+      helppos=dpos;
+      for(i=0,j=0; helpstr[i]!=0;i++) {
+        if(j==dline) break;
+        if(helpstr[i]==10) j++;
       }
-    } while(o==KEYNOP);
-  } while(o!=KEYESC && o!=KEYRESIZE);
+      if(j==dline) helpline=dline;
+      for(i=0,j=0; helpstr[i]!=0;i++) {
+        if(j==helpline) break;
+        if(helpstr[i]==10) j++;
+      }
+      for(;helpstr[i]!=0 && j<helpline+h;j++) {
+        for(l=0;helpstr[i+l]!=10 && helpstr[i+l]!=0;l++);
+        k=i+l+1;
+        drawsl("",j-helpline,0,MAX(0,-helppos));
+        l=MIN(l,l-helppos);
+        drawsl(helpstr+i+MAX(0,helppos),j-helpline,MAX(0,-helppos),l); 
+        drawsl("",j-helpline,l+MAX(0,-helppos),w);
+        i=k;
+      }   
+      for(;j<helpline+h;j++)
+        drawsl("",j-helpline,0,w);
+      dline=helpline;
+      dpos=helppos;
+      vgaupdatestatus();
+      do {
+        o=vgagetchar(1);
+        if('a'<= o && o <= 'z') o=o-'a'+'A';
+        switch(o) {
+        case KEYPREV: case 'I':
+          dline=MAX(0,dline-h/3); break;
+        case KEYNEXT: case 'M':
+          dline+=h/3; break;
+        case KEYUP: case 'U':
+          dline=MAX(0,dline-1); break;
+        case KEYDOWN: case 'N':
+          dline+=1; break;
+        case KEYRIGHT: case 'J':
+          dpos=MAX(0,MIN(dpos+w/3,HELPWIDTH-2.0/3*w)); break;
+        case KEYLEFT: case 'H':
+          dpos=MAX(-HELPWIDTH+2.0/3*w,dpos-w/3);
+        case KEYESC: case 'Q': break;
+        case KEYRESIZE: 
+          comdoresize();
+          break;
+        default: o=KEYNOP; break;
+        }
+      } while(o==KEYNOP);
+    } while(o!=KEYESC && o!='Q' && o!=KEYRESIZE);
+  } while(o!=KEYESC && o!='Q');
   drawstatusline();
 }
 
@@ -719,7 +725,11 @@ void comredraw(int mode){
     updatefilemk();
     vgaupdatestatus();
     pn=cpage->num;
-    killdvi(); 
+    killdvi();
+    if(numberargcount==1 && numberarg[0]==PGMAGIC) {
+      killfontdatabase();
+      initfontdatabase();
+    }
     initdvi();  
     comdoposit(pn,dvixpos,dviypos,fshrink,1);
     resetsearch();
@@ -1825,6 +1835,7 @@ void comdohypnext(void) {
     }
     if(readpage(cpage->num,fshrink,colors,0)) dodr=1;
     if(dodr) drawupdatepage();
+    else    drawshowallmarks();
   
     if(hypon) {
       comdomarks(xpxl-dvixpos,ypxl-dviypos);
@@ -1957,7 +1968,7 @@ main(int argc, char **argv) {
   long f;
 
 
-  pfprot( "This is some kind of a DVIewer, V 96.05\n");
+  pfprot( "This is some kind of a DVIewer, V 98.06\n");
   pfprot( "(C)opyright 1995 Thomas Moor\n\n");
 
   vgaopen();            /* open a window part 1 */
@@ -1982,6 +1993,7 @@ main(int argc, char **argv) {
   stralloccpy(&fontprefix,PKDIR);
   stralloccpy(&fontformat,PKNAME);
   stralloccpy(&tfmprefix,TFMDIR);
+  stralloccpy(&vfprefix,VFDIR);
   stralloccpy(&startupfilename,STARTUPFILENAME);
   /*env=getenv("DVIDRVFONTS");
   if(env!=NULL) stralloccpy(fontprefix,env);
@@ -2060,6 +2072,8 @@ main(int argc, char **argv) {
         stralloccpy(&fontformat, parm+2);  break;
       case 't':  case 'T':  
         stralloccpy(&tfmprefix, parm+2);  break;
+      case 'g':  case 'G':  
+        stralloccpy(&vfprefix, parm+2);  break;
       case 'm':  case 'M':  
         sscanf(parm+2, "%ld", &newmag);  break;
       case 'h':  case 'H':  
@@ -2089,6 +2103,7 @@ main(int argc, char **argv) {
         pfprot( " -f<font directory>      (default: %s)\n", fontprefix);
         pfprot( " -n<font name format>    (default: %s)\n", fontformat); 
         pfprot( " -t<TFMfile directory>   (default: %s)\n", tfmprefix);
+        pfprot( " -g<VFfile directory>    (default: %s)\n", vfprefix);
         pfprot( " -d<diplaywidth>x<height>(default: %dx%d pxl**2)\n",
                                                       vgaxdim,vgaydim);
         pfprot( " -m<new magnification>   (default: from DVIfile)\n");
@@ -2136,7 +2151,7 @@ main(int argc, char **argv) {
   
  
  
-  drawsl("TMVIEW V 96.05",0,0,vgastatuslen);
+  drawsl("TMVIEW V 98.06",0,0,vgastatuslen);
   drawsr("PRESS <?> TWICE FOR HELP",0,vgastatuslen-25,25);  
   vgaupdatestatus();
 
@@ -2275,20 +2290,24 @@ main(int argc, char **argv) {
     if(markon || hypon)
       vgasetmouse(MAX(0,MIN(vgaxdim-1,mousex)),MAX(0,MIN(vgaydim-1,mousey)));
     else 
-      vgasetmouse(vgaxdim/2,vgaydim/2);
+      vgasetmouse(vgaxdim/2,vgaydim/2); 
 #else
-    if(!(markon || hypon) && (mouseleft || mouseright)) {
-      vgasetmouse(vgaxdim/2,vgaydim/2);
-      ox=vgaxdim/2;
-      oy=vgaydim/2;
+    if(!(markon || hypon) &&  (mouseleft || mouseright)) {
+      if(mousex<20) ox=mousex+(20-mousex)/10+1;
+      if(mousex>vgaxdim-1-20) ox=mousex-(mousex-(vgaxdim-1-20))/10-1;
+      if(mousey<20) oy=mousey+(20-mousey)/10+1;
+      if(mousey>vgaydim-1-20) oy=mousey-(mousey-(vgaydim-1-20))/10-1;
     }
-    if(!(markon || hypon) && ((mouseleft && !ol) || (mouseright && !or))) {
-      mousex=vgaxdim/2;
-      mousey=vgaydim/2;
+    if( (markon || hypon) &&  (mouseleft || mouseright)) {
+      if(mousex<0) mousex=mousex/10-1;
+      if(mousex>vgaxdim-1) mousex=vgaxdim+(mousex-(vgaxdim-1))/10;
+      if(mousey<0) mousey=mousey/10-1;
+      if(mousey>vgaydim-1) mousey=vgaydim+(mousey-(vgaydim-1))/10;
+
     }
 #endif 
     if(markon) {                                          /* markon */
-      if(!mouseright) {
+      if(!mouseright && !mouseleft) {
         mousex=MAX(0,MIN(vgaxdim-1,mousex));
         mousey=MAX(0,MIN(vgaydim-1,mousey));
       }
@@ -2299,7 +2318,7 @@ main(int argc, char **argv) {
       if(mouseleft)              
         comdomarkfollow();
     } else { if(hypon) {                                  /* hypon */
-      if(!mouseright) {
+      if(!mouseright && !(mouseleft /* && !ol*/)) {
         mousex=MAX(0,MIN(vgaxdim-1,mousex));
         mousey=MAX(0,MIN(vgaydim-1,mousey));
       }
@@ -2329,10 +2348,8 @@ main(int argc, char **argv) {
       oy=vgaydim/2;
     }
 #else
-    if(!(markon || hypon)) {
-      ox=vgaxdim/2;
-      oy=vgaydim/2;
-    }
+    ox=mousex;
+    oy=mousey;
 #endif 
     ol=mouseleft;
     or=mouseright;
